/*By akemp (c) 2012
Released under the GNU BSD License.
If any of these files are used in a project or redistributed, please give credit to:
akemp
and link to:
https://sites.google.com/site/gamedev1988/
*/
#include <fstream>
#include <iostream>
#include <vector>
#include <algorithm>
#include <string>

using namespace std;

bool useFx = false;

struct point2d
{
    point2d (double x,double y)
    {
        this->x = x;
        this->y = y;
    };
    double x;
    double y;
};

struct point3d
{
    point3d (double x, double y, double z)
    {
        this->x = x;
        this->y = y;
        this->z = z;
    };
    double x;
    double y;
    double z;
};

struct pointvecd
{
    pointvecd (vector<double> w)
    {
        this->w = w;
    };
    vector<double> w;
};
struct objGroup
{
    string group;
    vector<point3d> v;
    vector<point2d> vt;
    vector<pointvecd> f;

    string groupString()
    {
        string groupStr = "g " + group + "\n";
        return groupStr;
    };
    string vString(int index)
    {
        if (index >= v.size() || index < 0)
        {
            cout << "An error has occurred: invalid index";
            return "";
        }
        string vStr = "v ";
        vStr += to_string((long double)v[index].x);
        vStr += " ";
        vStr += to_string((long double)v[index].y);
        vStr += " ";
        vStr += to_string((long double)v[index].z);
        vStr += "\n";
        return vStr;
    }
    string vtString(int i, int index)
    {
        if (i >= v.size() || i < 0)
        {
            cout << "An error has occurred: invalid index";
            return "";
        }
        string vStr = "vt ";
        if (useFx)
        {
            double xOffset = (double(index%8))/8 + 0.0625;
            double yOffset = 1-((double(index/8))/8 + 0.0625);
            vStr += to_string((long double)vt[i].x/100 + xOffset);
            vStr += " ";
            vStr += to_string((long double)vt[i].y/100 + yOffset);
            vStr += "\n";
        }
        else
        {
            vStr += to_string((long double)vt[i].x);
            vStr += " ";
            vStr += to_string((long double)vt[i].y);
            vStr += "\n";
        }
        return vStr;
    }
    string fString(int index)
    {
        if (index >= f.size() || index < 0)
        {
            cout << "An error has occurred: invalid index";
            return "";
        }
        string fStr;
        {
            fStr += "f ";
            for (int i = 0; i < f[index].w.size(); i++)
            {
                string w = to_string((long double)f[index].w[i]);
                fStr += w + "/" + w;
                fStr += " ";
            }
            fStr += "\n";
        }
        return fStr;
    }
    string allString(int index)
    {
        string allString = groupString();
        allString += "usemtl mat_" + group + "\n";
        for (int i = 0; i < v.size(); i++)
            allString += vString(i);
        for (int i =0; i < vt.size(); i++)
            allString += vtString(i, index);
        for (int i = 0; i < f.size(); i++)
            allString += fString(i);
        return allString;
    };
};

struct objFile
{
    vector<objGroup> group;
    
    string allString()
    {
        string allString;
        for (int i = 0; i < group.size(); i++)
            allString += group[i].allString(i) + "\n";
        return allString;
    }
};

struct faces
{
    string texture;
    vector<vector<point3d>> verts;
    vector<vector<point2d>> textures;
};

int main(int argc, char* argv[])
{
    ifstream fin;
    
    if (argc == 0)
    {
        cout << "Drag and drop a file to execute the utility.";
        return 1;
    }

    for (int m = 1; m < argc; m++)
    {
        fin.open(argv[m]);

        vector<faces> readIn;

        while (fin.good())
        {
            string texture;
            fin >> texture;
            for(int i = 0; i < texture.size(); i++)
            {
                if (texture[i] == '.')
                    texture[i] = '_';
            }
            if (texture != "")
            {
                int vertNum = 0;
                bool toggle = false;
                for (int i = 0; i < readIn.size(); i++)
                {
                    if (texture == readIn[i].texture)
                    {
                        fin >> vertNum;
                        vector<point3d> vertices;
                        vector<point2d> textures;
                        for (int i = 0; i < vertNum; i++)
                        {
                            double x, y, z;
                            fin >> x >> y >> z;
                            point3d pointer(x,y,z);
                            vertices.push_back(pointer);
                            fin >> x >> y;
                            point2d tex(x,y);
                            textures.push_back(tex);
                        }
                        readIn[i].verts.push_back(vertices);
                        readIn[i].textures.push_back(textures);
                        toggle = true;
                        break;
                    }
                }
                if (!toggle)
                {
                    faces tempFace;
                    tempFace.texture = texture;
                    fin >> vertNum;
                    vector<point3d> vertices;
                    vector<point2d> textures;
                    for (int i = 0; i < vertNum; i++)
                    {
                        double x, y, z;
                        fin >> x >> y >> z;
                        point3d pointer(x,y,z);
                        vertices.push_back(pointer);
                        fin >> x >> y;
                        point2d tpointer(x,y);
                        textures.push_back(tpointer);
                    }
                    tempFace.verts.push_back(vertices);
                    tempFace.textures.push_back(textures);
                    readIn.push_back(tempFace);
                }
            }
        }
        fin.close();
        objFile file;
        int count = 1;
        for(int i = 0; i < readIn.size(); i++)
        {
            objGroup temp;
            temp.group = readIn[i].texture;
            for (int j = 0; j < readIn[i].verts.size(); j++)
            {
                {
                    vector<double> verts;
                    for (int k = 0; k < readIn[i].verts[j].size(); k++)
                    {
                        temp.v.push_back(readIn[i].verts[j][k]);
                        verts.push_back(k + count);
                    }
                    for (int k = 0; k < readIn[i].textures[j].size(); k++)
                    {
                        temp.vt.push_back(readIn[i].textures[j][k]);
                    }
                    pointvecd face(verts);
                    temp.f.push_back(face);
                    count += readIn[i].verts[j].size();
                }
            }
            file.group.push_back(temp);
        }

        ofstream fout;
        string name = argv[m];
        for (int i = 0; i < name.size(); i++)
        {
            if (name[i] == '.')
                name[i] = '_';
        }
        string objName = name + ".obj";
        string mtlName = name + ".mtl";
        cout << "Outputting obj file to " << objName << endl;
        fout.open(objName);
        fout << "mtllib " + mtlName + "\n";
        fout << file.allString();
        fout.close();
        cout << "Output of obj file successful.\n Outputting mtl file.\n";
        fout.open(mtlName);
        for (int i = 0; i < file.group.size(); i++)
        {
            double bright = ((double)i)/file.group.size();
            fout << "newmtl mat_" + file.group[i].group + "\n";
            fout << "Ka " << bright << " " << bright << " " << bright << endl;
            fout << "Kd " << bright << " " << bright << " " << bright << endl;
            fout << "Ks " << bright << " " << bright << " " << bright << endl;
            fout << "Ns 0" << endl;
            fout << "map_Kd baseTex.tif" << endl;
        }
        fout.close();
        cout << "Output of mtl file successful.\n";
    }
    return 0;
}
